#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path
import copy

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest

import numpy as np


import imcut.dcmreaddata as dcmr

class LesionsTest(unittest.TestCase):

    @unittest.skip("Cekame, az to Tomas opravi")
    def test_import_lesion_editor(self):
        import lesioneditor
        import lesioneditor.Lession_editor_slim

    @unittest.skip("Cekame, az to Tomas opravi")
    def test_lesion_editor(self):
        """
        little more complex try to create editor object but do not run show 
        function

        """
        from PyQt4 import QtGui
        import lesioneditor
        import lesioneditor.Lession_editor_slim
        data3d = np.zeros([10, 11, 12], dtype=np.int16)
        segmentation = np.zeros(data3d.shape, dtype=np.int16)
        slab = {"liver": 1, "porta": 2}
        voxelsize_mm = [1.0, 1.2, 1.0]

        datap1={
            'data3d': data3d,
            'segmentation': segmentation,
            'slab': slab,
            'voxelsize_mm': voxelsize_mm
        }
        app = QtGui.QApplication(sys.argv)
        le = lesioneditor.Lession_editor_slim.LessionEditor(datap1=datap1)
    # @TODO znovu zprovoznit test

    @unittest.skip("Cekame, az to Tomas opravi")
    def test_synthetic_data_lesions_automatic_localization(self):
        """
        Function uses lesions  automatic localization in synthetic data.
        """
        import lesions
        #dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/')
# data
        slab = {'none':0, 'liver':1, 'porta':2, 'lesions':6}
        voxelsize_mm = np.array([1.0,1.0,1.2])

        segm = np.zeros([256,256,80], dtype=np.int16)

        # liver
        segm[70:190,40:220,30:60] = slab['liver']
# port
        segm[120:130,70:220,40:45] = slab['porta']
        segm[80:130,100:110,40:45] = slab['porta']
        segm[120:170,130:135,40:44] = slab['porta']

        # vytvoření kopie segmentace - před určením lézí
        segm_pre = copy.copy(segm)

        segm[150:180,70:105,42:55] = slab['lesions']


        data3d = np.zeros(segm.shape)
        data3d[segm== slab['none']] = 680
        data3d[segm== slab['liver']] = 1180
        data3d[segm== slab['porta']] = 1230
        data3d[segm== slab['lesions']] = 1110
        #noise = (np.random.rand(segm.shape[0], segm.shape[1], segm.shape[2])*30).astype(np.int16)
        noise = (np.random.normal(0,30,segm.shape))#.astype(np.int16)
        data3d = (data3d + noise  ).astype(np.int16)


        data={'data3d':data3d,
                'slab':slab,
                'voxelsize_mm':voxelsize_mm,
                'segmentation':segm_pre
                }

# @TODO je tam bug, prohlížeč neumí korektně pracovat s doubly
#        app = QApplication(sys.argv)
#        #pyed = QTSeedEditor(noise )
#        pyed = QTSeedEditor(data3d)
#        pyed.exec_()
#        #img3d = np.zeros([256,256,80], dtype=np.int16)

       # pyed = sed3.sed3(data3d)
       # pyed.show()

        tumory = lesions.Lesions()

        tumory.import_data(data)
        tumory.automatic_localization()
        #tumory.visualization()



# ověření výsledku
        #pyed = sed3.sed3(outputTmp, contour=segm==slab['porta'])
        #pyed.show()

        errim = np.abs(
                (tumory.segmentation == slab['lesions']).astype(np.int) -
                (segm == slab['lesions']).astype(np.int))

# ověření výsledku
        #pyed = sed3.sed3(errim, contour=segm==slab['porta'])
        #pyed.show()
#evaluation
        sum_of_wrong_voxels = np.sum(errim)
        sum_of_voxels = np.prod(segm.shape)

        #print "wrong ", sum_of_wrong_voxels
        #print "voxels", sum_of_voxels

        errorrate = sum_of_wrong_voxels/sum_of_voxels

        #import pdb; pdb.set_trace()

        self.assertLess(errorrate,0.1)




if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR% %USER% <%MAIL%>
#
# Distributed under terms of the %LICENSE% license.

"""
%HERE%
"""

import logging

logger = logging.getLogger(__name__)
import argparse
import PyQt4 #import QtGui, QtCore
import PyQt4.QtCore
import PyQt4.QtGui
import sys
import time

from . import lisa_data

def splash_screen(qapp):
    """
    create lisa splash screen
    :param qapp:
    :return:
    """
   # Create and display the splash screen
    lisa_data.create_lisa_data_dir_tree()
    splash_pix = PyQt4.QtGui.QPixmap(lisa_data.path('.lisa/LISA256.png'))
    splash = PyQt4.QtGui.QSplashScreen(splash_pix, PyQt4.QtCore.Qt.WindowStaysOnTopHint)
    splash.setMask(splash_pix.mask())
    splash.show()
    qapp.processEvents()
    return splash


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        # required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)
 # Simulate something that takes time

    app = PyQt4.QtGui.QApplication(sys.argv)
    splash = splash_screen(app)
    time.sleep(2)
    w = PyQt4.QtGui.QWidget()
    b = PyQt4.QtGui.QLabel(w)
    b.setText("Hello World!")
    w.setGeometry(100,100,200,50)
    b.move(50,20)
    w.setWindowTitle("PyQt")
    w.show()
    splash.finish(w)
    app.exec_()
if __name__ == "__main__":
    main()


#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
import unittest
import os.path as op
from nose.plugins.attrib import attr

path_to_script = op.dirname(op.abspath(__file__))

import sys

sys.path.insert(0, op.abspath(op.join(path_to_script, "../../io3d")))
sys.path.insert(0, op.abspath(op.join(path_to_script, "../../imma")))
# import sys
# import os.path

# imcut_path =  os.path.join(path_to_script, "../../imcut/")
# sys.path.insert(0, imcut_path)
import lisa.virtual_resection
import numpy as np


class OrganSegmentationImageManipulationTest(unittest.TestCase):

    def test_relabel_organ_segmentation(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        slab = datap["slab"]
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        self.assertGreater(np.sum(oseg.select_label("porta")), 0, "Generated porta should be bigger than 0 voxels")
        oseg.segmentation_relabel("porta", "liver")

        self.assertEqual(np.sum(oseg.select_label("porta")), 0)
        self.assertGreater(np.sum(oseg.select_label("liver")), 100)

    def test_relabel_organ_segmentation_from_multiple_labels(self):
        import lisa.organ_segmentation
        import io3d
        # datap = io3d.datasets.generate_abdominal()
        datap = io3d.datasets.generate_synthetic_liver(return_dataplus=True)
        slab = datap["slab"]
        slab["tumor"] = 3
        datap["segmentation"][5:10, 5:10, 5:10] = slab["tumor"]
        oseg = lisa.organ_segmentation.OrganSegmentation()
        oseg.import_dataplus(datap)
        self.assertGreater(np.sum(oseg.select_label("porta")), 0, "Generated porta should be bigger than 0 voxels")
        oseg.segmentation_relabel(["porta", "tumor"], "liver")

        self.assertEqual(np.sum(oseg.select_label("porta")), 0)
        self.assertEqual(np.sum(oseg.select_label("tumor")), 0)
        self.assertGreater(np.sum(oseg.select_label("liver")), 100)


if __name__ == "__main__":
    # logging.basicConfig(stream=sys.stderr)
    logger.setLevel(logging.DEBUG)
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 mjirik <mjirik@mjirik-Latitude-E6520>
#
# Distributed under terms of the MIT license.

"""

"""

import logging
logger = logging.getLogger(__name__)
import argparse

import web


class LisaServer:
    def __init__(self):
        """TODO: Docstring for __import__.

        :arg1: TODO
        :returns: TODO

        """
        self.urls = ('/(.*)', 'hello')
        self.app = web.application(self.urls, globals())

    def run(self):
        self.app.run()


class hello:
    def __init__(self):
        # self.oseg = osg.OrganSegmentation()
        pass

    def process_msg(self, data):
        if not data:
            data = 'abs(50)'
        web.header('Access-Control-Allow-Origin',      '*')
        web.header('Access-Control-Allow-Credentials', 'true')
        retval = \
            "<!DOCTYPE html><html><body><div>" + \
            data +\
            "</div></body></html>"
        # "<b>" + str(eval(data)) + "</b>" +
        str(eval(data))
        return str(retval)

    def GET(self, data):
        return self.process_msg(data)

    def POST(self, data):
        return self.process_msg(data)


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    server = LisaServer()
    server.run()


if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 mjirik <mjirik@mjirik-HP-Compaq-Elite-8300-MT>
#
# Distributed under terms of the MIT license.

"""

"""
import unittest
from nose.plugins.attrib import attr
import io3d
import numpy as np
import sys
import os

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../../imcut/src"))

import os
path_to_script = os.path.dirname(os.path.abspath(__file__))
import logging
logger = logging.getLogger(__name__)

import lisa.dataset
from lisa import organ_segmentation
from lisa import organ_model

class LiverSegmentationTest(unittest.TestCase):

    # test se pouští ze složky lisa
    # nosetests tests/liver_segmentation_test.py -a actual

    @attr('interactive')
    @attr('slow')
    def test_automatic(self):
        pass

    @attr('slow')
    def test_train_liver_model(self):
        fn_mdl = "test_liver_intensity.Model.p"
        organ_model.train_liver_model_from_sliver_data(
            output_file= fn_mdl,
            sliver_reference_dir="~/lisa_data/sample_data/",
            orig_pattern="*orig*01.mhd",
            ref_pattern="*seg*01.mhd",
        )


    @attr('slow')
    def test_train_liver_model_and_liver_segmentation(self):
        """
        combination of test_train_liver_model and test_organ_model_with_pretrained_classifier
        :return:
        """
        fn_mdl = "test_liver_intensity.Model.p"
        organ_model.train_liver_model_from_sliver_data(
            output_file=fn_mdl,
            sliver_reference_dir="~/lisa_data/sample_data/",
            orig_pattern="*orig*01.mhd",
            ref_pattern="*seg*01.mhd",
        )

        path_to_data = lisa.dataset.sample_data_path()
        dcmdir = os.path.join(path_to_data, './liver-orig001.mhd')

        print("Interactive test: with left mouse button select liver, \
            with right mouse button select other tissues")
        # gcparams = {'pairwiseAlpha':10, 'use_boundary_penalties':True}
        # fn_mdl = os.path.expanduser("~/lisa_data/liver_intensity.Model.p")
        segparams = {}
        segmodelparams={
            'mdl_stored_file': fn_mdl,
            'fv_type':'fv_extern',
            'fv_extern': "intensity_localization_fv"
        }
        # 'pairwise_alpha_per': 3,
        #              'use_boundary_penalties': True,
        #              'boundary_penalties_sigma': 200}
        oseg = organ_segmentation.OrganSegmentation(
            dcmdir, working_voxelsize_mm=4, segparams=segparams, segmodelparams=segmodelparams)
        oseg.add_seeds_mm([200], [110, 150], [110, 100], label=1, radius=30, width=5)
        oseg.add_seeds_mm([200], [250, 230], [210, 260], label=2, radius=50, width=5)

        # from PyQt4.QtGui import QApplication
        # app = QApplication(sys.argv)
        # oseg.interactivity()
        oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()

        # misc.obj_to_file(oseg.get_iparams(),'iparams.pkl', filetype='pickle')

        self.assertGreater(volume, 1300000)
        self.assertLess(volume, 2100000)

    # @unittest.skipIf(not interactiveTest, "interactive test")
    # @unittest.skip("interactivity params are obsolete")

    @attr('interactive')
    @attr('slow')
    def test_organ_segmentation_with_pretrained_classifier(self):
        """
        Interactivity is stored to file
        :rtype: object
        """

        path_to_data = lisa.dataset.sample_data_path()
        dcmdir = os.path.join(path_to_data, './liver-orig001.mhd')

        print("Interactive test: with left mouse button select liver, \
            with right mouse button select other tissues")
        # gcparams = {'pairwiseAlpha':10, 'use_boundary_penalties':True}
        fn_mdl = os.path.expanduser("~/lisa_data/liver_intensity.Model.p")
        segparams = {}
        segmodelparams={
            'mdl_stored_file': fn_mdl,
            'fv_type':'fv_extern',
            'fv_extern': "intensity_localization_fv"
        }
        # 'pairwise_alpha_per': 3,
        #              'use_boundary_penalties': True,
        #              'boundary_penalties_sigma': 200}
        oseg = organ_segmentation.OrganSegmentation(
            dcmdir, working_voxelsize_mm=4, segparams=segparams, segmodelparams=segmodelparams)
        oseg.add_seeds_mm([200], [110, 150], [110, 100], label=1, radius=30, width=5)
        oseg.add_seeds_mm([200], [250, 230], [210, 260], label=2, radius=50, width=5)

        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        oseg.interactivity()
        # oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()

        # misc.obj_to_file(oseg.get_iparams(),'iparams.pkl', filetype='pickle')

        self.assertGreater(volume, 1000000)
        self.assertLess(volume, 2000000)

    @attr('interactive')
    @attr('slow')
    def test_automatic_liver_seeds(self):
        """
        Interactivity is stored to file
        :rtype: object
        """

        path_to_data = lisa.dataset.sample_data_path()
        dcmdir = os.path.join(path_to_data, './liver-orig001.mhd')

        print("Interactive test: with left mouse button select liver, \
            with right mouse button select other tissues")
        # gcparams = {'pairwiseAlpha':10, 'use_boundary_penalties':True}
        fn_mdl = os.path.expanduser("~/lisa_data/liver_intensity.Model.p")
        segparams = {}
        segmodelparams={
            # 'mdl_stored_file': fn_mdl,
            # 'fv_type':'fv_extern',
            # 'fv_extern': "intensity_localization_fv"
        }
        # 'pairwise_alpha_per': 3,
        #              'use_boundary_penalties': True,
        #              'boundary_penalties_sigma': 200}
        oseg = organ_segmentation.OrganSegmentation(
            dcmdir, working_voxelsize_mm=4, segparams=segparams, segmodelparams=segmodelparams)
        # oseg.add_seeds_mm([110, 150], [110, 100], [200], label=1, radius=30, width=5)
        # oseg.add_seeds_mm([250, 230], [210, 260], [200], label=2, radius=50, width=5)
        print(oseg.seeds)
        oseg.automatic_liver_seeds()

        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        oseg.interactivity()
        # oseg.ninteractivity()

        volume = oseg.get_segmented_volume_size_mm3()

        # misc.obj_to_file(oseg.get_iparams(),'iparams.pkl', filetype='pickle')

        self.assertGreater(volume, 1000000)
        self.assertLess(volume, 2000000)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import sys
import os.path
import numpy as np
# from scipy import signal
import matplotlib.pyplot as plt
# import skimage.exposure as skexp

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
# import featurevector

import logging
logger = logging.getLogger(__name__)
import argparse

import scipy

# ----------------- my scripts --------
import misc
import sed3

try:
    import dcmreaddata
except:
    from imcut import dcmreaddata


# import show3
# import vessel_cut
import scipy.ndimage.measurements as scimeas
from skimage.morphology import remove_small_objects
import skimage.morphology as skimor
from skimage.segmentation import random_walker
# import skimage.filter as skifil
# from skimage import measure as skmeasure
import scipy.ndimage.morphology as scimorph
# from scipy.ndimage import generate_binary_structure
# import cv2
import tools
# from mayavi import mlab
# import pymorph as pm

# version-dependent imports ---------------------------------
from pkg_resources import parse_version

# importing distance transform
if parse_version(scipy.__version__) > parse_version('0.9'):
    from scipy.ndimage.morphology import distance_transform_edt
else:
    from scipy.ndimage import distance_transform_edt
# -----------------------------------------------------------


class Lesions:
    """

    lesions = Lesions(data3d, voxelsize_mm, segmentation, slab)
    lesions.automatic_localization()

    or

    lesions = Lesions()
    lesions.import_data(data)
    lesions.automatic_localization()




    """

# ----------------------------------------------------------------------------
# ---------------------------------------------------------------------------


    def __init__(self, data3d=None, voxelsize_mm=None, segmentation=None,
                 slab=None):


        self.data3d = data3d
        self.data = None
        self.segmentation = segmentation
        self.slab = slab
        self.voxelsize_mm = voxelsize_mm
        # min size of object to be considered as lession
        # self.min_size_of_comp = 200
        self.min_size_of_comp = 50

# ---------------------------------------------------------------------------
    def run_gui(self):
        import lesioneditor
        import lesioneditor.Lession_editor_slim
        datap1={
            'data3d': self.data3d,
            'segmentation': self.segmentation,
            'slab': self.slab,
            'voxelsize_mm': self.voxelsize_mm
        }
        le = lesioneditor.Lession_editor_slim.LessionEditor(datap1=datap1)
        le.show()
        # get data
        # self.segmentation = le.segmentation
        # self.slab = le.slab

       
# ---------------------------------------------------------------------------
    def import_data(self, data):
        self.data = data
        self.data3d = data['data3d']
        self.segmentation = data['segmentation']
        self.slab = data['slab']
        self.voxelsize_mm = data['voxelsize_mm']

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def automatic_localization(self):
        """
        Automatic localization of lesions. Params from constructor
        or import_data() function.
        """
        self._automatic_localization()

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def export_data(self):
        self.data['segmentation'] = self.segmentation
        pass

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def _automatic_localization(self):
        """
        Automatic localization made by Tomas Ryba.
        """

        # seeds = self.get_seeds_using_class_1(class1)
        liver = self.data3d * (self.segmentation != 0)
        print('analyzing histogram...')
        class1 = tools.analyse_histogram(self.data3d,
                                         roi=self.segmentation != 0)
        # sed3.sed3(self.data3d, seeds=class1).show()
        print('getting seeds...')
        seeds = self.get_seeds_using_prob_class1(
            liver,
            class1,
            thresholdType='percOfMaxDist',
            percT=0.3)

        # sed3.sed3(self.data3d, seeds=seeds).show()

        print('Starting random walker...')
        rw = random_walker(liver, seeds, mode='cg_mg')
        print('...finished.')

        label_l = self.data['slab']['lesions']

        lessions = rw == 2
        sed3.sed3(self.data3d, contour=lessions).show()
        lessions = self.filter_objects(lessions)

        self.segmentation = np.where(lessions, label_l, self.segmentation)

        # self.mayavi3dVisualization()

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def remove_objects_with_label(self, objects, label):
        """
        This function removes all objects from <objects> that contains a
        voxel that is labeled as <label>.
        """
        labels, nlabels = self.get_objects(objects)
        labeled_objs = self.segmentation == label
        not_vessel_objects = np.zeros_like(self.segmentation)

        # iterates through all founded objects
        for l in range(1, nlabels+1):
            obj = labels == l
            # count number of voxels that are also labeled ass vessels
            same_voxs = np.logical_and(labeled_objs, obj).sum()
            # if there are no vessel voxels, then this object is taken into
            # account
            if same_voxs == 0:
                not_vessel_objects += obj

        return not_vessel_objects

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_objects(self, objects=np.zeros(1, dtype=np.bool)):
        if not objects.any():
            label_l = self.data['slab']['l esions']
            objects = self.segmentation == label_l

        only_big = remove_small_objects(objects,
                                        min_size=self.min_size_of_comp,
                                        connectivity=1, in_place=False)
        labels, nlabels = scimeas.label(only_big)

        return labels, nlabels

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_features(self, labels, nlabels):
        features = np.zeros((nlabels, 2))
        for lab in range(1, nlabels+1):
            obj = labels == lab
            size = obj.sum()
            strel = np.ones((3, 3, 3), dtype=np.bool)
            # obj =scimorph.binary_closing(obj, generate_binary_structure(3,3))
            obj = scimorph.binary_closing(obj, strel)
            compactness = self.get_zunics_compatness(obj)
            features[lab-1, 0] = size
            features[lab-1, 1] = compactness

            print('size = %i' % size)
            print('compactness = %.3f' % compactness)
            # sed3.sed3(self.data3d, contour=(labels==lab)).show() # noqa
        print(features[:, 1])
        return features

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def filter_objects(self, objects, min_comp=0.3):
        """
        This function removes objects that aren't lessions.
        Firstly, it removes objects, that contains voxels labeled as vessels.
        Then it filters out to small objects and finally it calculates object
        features and removes objects with bad features (e.g. to non-compact
        etc.).
        """
        # TODO: If a lession is near a vessel, it is possible that the lession
        # will have a voxel labeled as vessel and will be filtered out.
        # removing objects that contains voxels labeled as porta
        if 'porta' in self.data['slab']:
            objects = self.remove_objects_with_label(
                objects,
                label=self.data['slab']['porta'])

        # removing to small objects
        print('Small objects removal:')
        _, nlabels = scimeas.label(objects)
        print('    before: nlabels = %i' % nlabels)
        objects = skimor.remove_small_objects(
            objects, min_size=self.min_size_of_comp,
            connectivity=1, in_place=False)
        _, n_labels = scimeas.label(objects)
        print('    after: nlabels = %i' % nlabels)

        # computing object features
        labels, n_labels = scimeas.label(objects)
        features = self.get_features(labels, n_labels)

        print('Non-compact objects removal:')
        print('    before: nlabels = %i' % n_labels)
        # filtering objects with respect to their features
        features_ok = features[:, 1] >= min_comp
        objs_ok = np.zeros_like(objects)
        for i in np.argwhere(features_ok > 0):
            objs_ok += labels == (i + 1)
        print('\tafter: nlabels = %i' % features_ok.sum())

        return objs_ok

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_bounds(self, obj):
        obj_er = scimorph.binary_erosion(obj)
        bounds = obj - obj_er
        return bounds

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_moment(self, obj, p, q, r):
        elems = np.argwhere(obj)
        mom = 0
        for el in elems:
            mom += el[0]**p + el[1]**q + el[2]**r
        return mom

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_central_moment(self, obj, p, q, r):
        elems = np.argwhere(obj)
        m000 = obj.sum()
        m100 = (elems[:, 0]).sum()
        m010 = (elems[:, 1]).sum()
        m001 = (elems[:, 2]).sum()
        xc = m100 / m000
        yc = m010 / m000
        zc = m001 / m000

        mom = 0
        for el in elems:
            mom += (el[0] - xc)**p + (el[1] - yc)**q + (el[2] - zc)**r

        # mxyz = elems.sum(axis=0)
        # cent = mxyz / m000
        # mom = 0
        # for el in elems:
        #     mom += (el[0] - cent[0])**p + (el[1] - cent[1])**q +
        #                                    (el[2] - cent[2])**r
        return mom

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_zunics_compatness(self, obj):
        m000 = obj.sum()
        m200 = self.get_central_moment(obj, 2, 0, 0)
        m020 = self.get_central_moment(obj, 0, 2, 0)
        m002 = self.get_central_moment(obj, 0, 0, 2)
        term_1 = (3**(5./3)) / (5 * (4*np.pi)**(2./3))
        term_2 = m000**(5./3) / (m200 + m020 + m002)
        k = term_1 * term_2
        return k

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    # def mayavi3dVisualization(self, objs = 'pvl'):
    #     data = np.zeros_like(self.data3d)
    #     srcs = list()
    #     colors = list()
    #     if 'p' in objs:
    #         #liver = self.segmentation == self.data['slab']['liver']
    #         #parenchym = np.logical_or(liver, vessels)
    #         parenchym = self.segmentation > 0
    #         data = np.where(parenchym, 1, 0)
    #         data = data.T
    #         srcs.append(mlab.pipeline.scalar_field(data))
    #         colors.append((0,1,0))
    #     if 'v' in objs:
    #         vessels = self.segmentation == self.data['slab']['porta']
    #         data = np.where(vessels, 1, 0)
    #         data = data.T
    #         srcs.append(mlab.pipeline.scalar_field(data))
    #         colors.append((1,0,0))
    #     if 'l' in objs:
    #         lessions = self.segmentation == self.data['slab']['lesions']
    #         data = np.where(lessions, 1, 0)
    #         data = data.T
    #         srcs.append(mlab.pipeline.scalar_field(data))
    #         colors.append((0,0,1))
    #
    #     for src, col in zip(srcs, colors):
    #         src.spacing = [.62, .62, 5]
    #         mlab.pipeline.iso_surface(src, contours=2, opacity=0.1,color=col)
    #
    #     mlab.show()

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def visualization(self):
        pyed = sed3.sed3(
            self.segmentation == self.data['slab']['lesions'])
        pyed.show()

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_seeds_using_class_1(self, class1):
        dist_data = np.where(class1 == 1, False, True)
        dist_data *= self.segmentation > 0
        dists = distance_transform_edt(dist_data)

        seeds = dists > 0.5 * dists.max()

        all_seeds = np.zeros(self.data3d.shape, dtype=np.int)
        # allSeeds[np.nonzero(self.segmentation)] = 80
        all_seeds[np.nonzero(class1)] = 1  # zdrava tkan
        all_seeds[np.nonzero(seeds)] = 2  # outliers
        # kvuli segmentaci pomoci random walkera se omezi obraz pouze na
        # segmentovana jatra a cevy
        all_seeds = np.where(self.segmentation == 0, -1, all_seeds)

        return all_seeds

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def get_seeds_using_prob_class1(self, data, class1, roi=None, dens_min=20,
                                    dens_max=255,
                                    thresholdType='percOfMaxDist', percT=0.5):
        # calculates probability based on similarity of intensities
        probs, mu = tools.intensity_probability(data, std=10)
        # sed3.sed3(data).show()
        # sed3.sed3(probs).show()
        # normalizing and calculating reciprocal values
        # weights_ints = skexp.rescale_intensity(probs,
        # in_range=(0,probs.max()), out_range=(1,0))
        weights_ints = np.exp(-probs)

        weights_h = np.where(data > mu, 1 - probs, 0)
        weights_l = np.where(data < mu, 1 - probs, 0)
        # sed3.sed3(1 - probs).show()
        sed3.sed3(weights_h).show()
        sed3.sed3(weights_l).show()

        if roi is None:
            roi = np.logical_and(data >= dens_min, data <= dens_max)
        dist_data = np.where(class1 == 1, False, True)
        dist_data *= roi > 0
        # dists = distance_transform_edt(dist_data)
        # sed3.sed3(dists).show()

        # print 'dists max = %i' % dists.max()
        # print 'dists min = %i' % dists.min()
        # print 'weights_ints max = %.4f' % weights_ints.max()
        # print 'weights_ints min = %.4f' % weights_ints.min()
        # print 'probs max = %.4f' % probs.max()
        # print 'probs min = %.4f' % probs.min()

        # energy = dists * weights_ints
        energy = weights_ints
        # sed3.sed3(energy).show()

        seeds = np.zeros(data.shape, dtype=np.bool)
        if thresholdType == 'percOfMaxDist':
            seeds = energy > (percT * energy.max())
        elif thresholdType == 'mean':
            seeds = energy > 2 * (energy[np.nonzero(energy)]).mean()

        # TODO: tady je problem, ze energy je v intervalu <0.961, 1> - hrozne
        # maly rozsah
        print('energy max = %.4f' % energy.max())
        print('energy min = %.4f' % energy.min())
        print('thresh = %.4f' % (percT * energy.max()))
        print(seeds.min())
        print(seeds.max())
        print('seed perc = %.2f' % (
            (energy > percT * energy.max()).sum()/np.float(energy.nbytes)))
        sed3.sed3(seeds).show()

        # removing to small objects
        min_size_of_seed_area = 60
        print('before removing: %i' % seeds.sum())
        seeds = skimor.remove_small_objects(
            seeds, min_size=min_size_of_seed_area, connectivity=1,
            in_place=False)
        print('after removing: %i' % seeds.sum())

        all_seeds = np.zeros(data.shape, dtype=np.int)
        # allSeeds[np.nonzero(self.segmentation)] = 80
        all_seeds[np.nonzero(class1)] = 1  # zdrava tkan
        all_seeds[np.nonzero(seeds)] = 2  # outliers
        # kvuli segmentaci pomoci random walkera se omezi obraz pouze na
        # segmentovana jatra a cevy
        all_seeds = np.where(roi == 0, -1, all_seeds)

        sed3.sed3(all_seeds).show()

        return all_seeds

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
    def overlay_test(self):
        # filename =
        # r'c:\Data\kky\pro mechaniku 21.12.2012\53009707\Export0000\SR0000'
        # filename = r'c:\Data\kky\pro mechaniku
        # 21.12.2012\52496602\Export0001\SR0000'
        # filename = r'c:\Data\kky\pro mechaniku
        # 21.12.2012\54559730\Export0000\SR0000'
        filename = r'c:\Data\kky\pro mechaniku 21.12.2012\53596059\Export0000\SR0000' # noqa
        # dcr = dcmreaddata.DicomReader(
        # r'c:\Data\kky\pro mechaniku 21.12.2012\52496602\Export0001\SR0000')
        dcr = dcmreaddata.DicomReader(filename)
        data3d = dcr.get_3Ddata()
        overlay = dcr.get_overlay()
        win_w = 350
        win_c = 50
        min_w = win_c - (win_w / 2.)
        max_w = win_c + (win_w / 2.)

        for key in overlay.keys():
            overlay = overlay[key]

            for i in range(overlay.shape[0]):
                plt.figure(), plt.gray()
                plt.subplot(121), plt.imshow(data3d[i, :, :], vmin=min_w,
                                             vmax=max_w)
                plt.subplot(122), plt.imshow(overlay[i, :, :])
                cv2.imwrite(filename + '/overlay_%i.png' % i, data3d[i, :, :])

        plt.show()


# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------
def main():

    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
        description='Module for segmentation of simple anatomical structures')
    parser.add_argument('-i', '--inputfile', default='vessels.pkl',
                        help='path to data dir')

    # args = parser.parse_args()
    # horsi kvalita segmentace
    # datapath = os.path.join(path_to_script, "../vessels1.pkl")
    # hypodenzni meta
    # datapath = os.path.join(path_to_script, args.inputfile)
    # horsi kvalita segmentace
    # datapath = os.path.join(path_to_script, "../organ.pkl")

    # data = misc.obj_from_file(args.inputfile, filetype = 'pickle')

    dcmdir = '/home/tomas/Dropbox/Work/Data/medical/org-38289898-export1.pklz'
    data = misc.obj_from_file(dcmdir, filetype='pickle')

    # windowing
    data['data3d'] = tools.windowing(data['data3d'], level=50, width=350)
    # data['data3d'] = smoothing(data['data3d'], sliceId=0)

    # smoothing ----------------
    # bilateral
    # data['data3d'] = tools.smoothing_bilateral(data['data3d'],
    # sigma_space=15, sigma_color=0.05, sliceId=0)
    # more bilateral
    # data['data3d'] = tools.smoothing_bilateral(data['data3d'],
    # sigma_space=15, sigma_color=0.1, sliceId=0)
    # total-variation
    data['data3d'] = tools.smoothing_tv(data['data3d'], weight=0.05, sliceId=0)
    # more total-variation
    # data['data3d'] = tools.smoothing_tv(data['data3d'], weight=0.2,
    # multichannel=False, sliceId=0)
    # sed3.sed3(data['data3d']).show()

    tumory = Lesions()
    # tumory.overlay_test()
    tumory.import_data(data)
    tumory.automatic_localization()

    tumors = tumory.segmentation == tumory.data['slab']['lesions']
    sed3.sed3(tumory.data3d, contour=tumors).show()

#    SectorDisplay2__()

if __name__ == "__main__":
    main()



#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
# sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector

import logging
logger = logging.getLogger(__name__)


#import apdb
#  apdb.set_trace();\
#import scipy.io
import numpy as np
import scipy

# ----------------- my scripts --------
try:
    import dcmreaddata as dcmr
except:
    from imcut import dcmreaddata as dcmr

# try:
#     from imcut import pycut
# except:
#     logger.warning("Deprecated of pyseg_base as submodule")
#     import pycut
import argparse
import io3d
import io3d.misc
import scipy.ndimage.filters as filters
import numpy as np
from scipy.ndimage.measurements import label
from scipy.ndimage import morphology


class SupportStructureSegmentation():
    def __init__(self,
            data3d = None,
            voxelsize_mm = None,
            autocrop = True,
            autocrop_margin_mm = [10,10,10],
            modality = 'CT',
            slab = {'none':0,'llung':4, 'bone':8,'rlung':9,'heart':6,'diaphragm':5},
            maximal_lung_diff = 0.4,
            rad_diaphragm=4,
            smer=None,
            working_voxelsize_mm = [2.0, 2.0, 2.0]
        ):

        """
        Segmentaton of support structures for liver segmentatio based on
        locati0on prior.
        """



        self.orig_voxelsize_mm = voxelsize_mm
        self.orig_shape = data3d.shape
        self.working_voxelsize_mm = working_voxelsize_mm
        self.crinfo = [[0,-1],[0,-1],[0,-1]]
        self.slab = slab
        self.maximal_lung_diff = maximal_lung_diff
        self.rad_diaphragm=rad_diaphragm
        self.smer=smer


        # data resize
        self.data3d = io3d.misc.resize_to_mm(data3d, voxelsize_mm, working_voxelsize_mm)
        self.segmentation = np.zeros(self.data3d.shape , dtype=np.int8)
        self.resized = True
        # self.data3d = io3d.misc.resize_to_mm(data3d, voxelsize_mm, working_voxelsize_mm)
        self.voxelsize_mm = self.working_voxelsize_mm

        self.autocrop = autocrop
        self.autocrop_margin_mm = np.array(autocrop_margin_mm)
        self.autocrop_margin = self.autocrop_margin_mm/self.voxelsize_mm

        #import pdb; pdb.set_trace()

    def resize_back_to_orig(self):
        """
        Resize segmentation to original shape
        """
        if self.resized:
            self.segmentation = io3d.misc.resize_to_shape(self.segmentation,
                                                      self.orig_shape)
            self.resized = False


    def run(self):
        logger.debug('bones')
        self.bone_segmentation()
        logger.debug('spine')
        self.spine_segmentation()
        logger.debug('lungs')
        self.lungs_segmentation()
        logger.debug('heart')
        self.heart_segmentation()

        self.resize_back_to_orig()



    def import_data(self, data):
        self.data = data
        self.data3d = data['data3d']
        self.voxelsize_mm = data['voxelsize_mm']

    def import_dir(self, datadir):
        reader = dcmr.DicomReader(datadir)
        self.data3d = reader.get_3Ddata()
        self.metadata = reader.get_metaData()
        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(datadir)
        self.voxelsize_mm = np.array(self.metadata['voxelsize_mm'])


    def convolve_structure_heart( self , size=9 ):
        a = np.zeros(( size , size , size ))
        c = int (np.floor( size / 2 ))
        a[c,c,c]=1
        structure = filters.gaussian_filter( a , self.voxelsize_mm )
        structure[:c,:,:] *= -1
        structure[c,:,:] = 0
        return structure

    

    def bone_segmentation(self, bone_threshold = 200):
#ipdb.set_trace()
        return np.array(self.data3d > bone_threshold)#.astype(np.int8)*self.slab['bone']

    def convolve_structure_spine(self, velikost = [300 , 2, 2]):
        structure = np.ones((int(velikost[0]/self.voxelsize_mm[0]),int(velikost[1]/self.voxelsize_mm[1]),int(velikost[2]/self.voxelsize_mm[2])))  
        return structure

    

    def spine_segmentation(self, bone_threshold= 330):
        seg_prub = filters.gaussian_filter(self.data3d, 5.0/np.asarray(self.voxelsize_mm))
        structure = self.convolve_structure_spine()
        
        seg_prub = filters.convolve(seg_prub, structure)
	
	
#seg_prub = scipy.signal.fftconvolve(seg_prub, self.convolve_structure_spine())
        maximum = np.amax(seg_prub)
        seg_prub = np.array(seg_prub > 0.55*maximum)
#seg_prub = seg_prub * np.array(self.data3d>=bone_threshold)
#import sed3
#ed = sed3.sed3(seg_prub)
#ed.show()
        self.segmentation = seg_prub
        pass


    # def __spl(self, x,y):
    #     return interpolate.bisplev(x,y, tck)

    def __above_diaphragm_calculation(self, seg_prub, internal_resize_shape=[20, 20, 20], data_degradation=4):
        import multipolyfit as mpf
        # print seg_prub.dtype
        # seg_prub_tmp = misc.resize_to_shape(seg_prub, internal_resize_shape)
        # print seg_prub_tmp.dtype
        seg_prub_tmp = seg_prub
        z, x, y = np.nonzero(seg_prub_tmp)
# data degradation
        x = x[::data_degradation]
        y = y[::data_degradation]
        z = z[::data_degradation]
        # import PyQt4.QtCore
        # PyQt4.QtCore.pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        s = np.array([x , y]).T
        h = np.array(z)
        model = mpf.multipolyfit(s, h, self.rad_diaphragm, model_out = True)
        # tck = interpolate.bisplrep(x,y,z, s=10)
        ran = seg_prub.shape
        x = np.arange( ran[1] )
        y = np.arange( ran[2] )
        x, y = np.meshgrid( x, y)
        # sh = [len(x), len(y)]
##x, y = np.meshgrid(x, y)
        # z = np.asarray(map(self.__spl(x,y), x.reshape(-1), y.reshape(-1))).reshape(sh)


        z = np.floor(np.asarray(map(model, x.reshape(-1), y.reshape(-1)))).astype(int)
        x = x.reshape(z.shape)
        y = y.reshape(z.shape)
        cc = np.zeros(ran)
        for a in range(z.shape[0]):
            if (z[a] < ran[0]) and (z[a] >= 0):

                #self.segmentation[z[a], x[a], y[a]] = self.slab['diaphragm']
                if self.smer==0:
                    cc[z[a]+1:, x[a], y[a]] = 1
                else:
                    cc[:z[a]-1, x[a], y[a]] = 1

                # self.segmentation[z[a], x[a], y[a]] = self.slab['diaphragm']
                for b in range(z[a]+1, ran[0]):
                    cc[b, x[a], y[a]] = 1

        # cc = misc.resize_to_shape(cc, seg_prub.shape)
        return cc

    def heart_segmentation(self, heart_threshold = 0, top_threshold = 200):
        a=self.convolve_structure_heart()
        seg_prub = np.array(self.segmentation == self.slab['rlung'])+np.array(self.segmentation == self.slab['llung'])


        logger.debug('pred konvoluci')
        seg_prub = filters.convolve( (seg_prub-0.5) , a )
        logger.debug('po konvoluci')


# import sed3
# ed = sed3.sed3(seg_prub)
# ed.show()
        if self.smer==0:
            seg_prub = np.array(seg_prub<=-0.3)
        else:
            seg_prub = np.array(seg_prub<=0.3)
        cc = self.__above_diaphragm_calculation(seg_prub)
#ipdb.set_trace()
        plice1=np.array(self.segmentation==self.slab['llung'])
        z, x, y = np.nonzero(plice1)

        x1 = [0,0,0,0]
        y1 = [0,0,0,0]
        z1 = [0,0,0,0]
        x1[0]=np.min(x)
        x1[1]=np.max(x)
        y1[0]=np.min(y)
        y1[1]=np.max(y)
        z1[0]=np.max(z)
        z1[1]=np.min(z)
        plice2=np.array(self.segmentation==self.slab['rlung'])
        z, x, y = np.nonzero(plice2)
        x1[2]=np.min(x)
        x1[3]=np.max(x)
        y1[2]=np.min(y)
        y1[3]=np.max(y)
        z1[2]=np.max(z)
        z1[3]=np.min(z)
        mp=np.zeros(self.segmentation.shape)
        xmin=np.min(x1)
        xmax=np.max(x1)
        ymin=np.min(y1)
        ymax=np.max(y1)
        zmin=np.min(z1)
        zmax=np.max(z1)
        if self.smer==0:
            mp[zmin:,  xmin:xmax ,ymin:ymax]=1
        else:
            mp[:zmax,  xmin:xmax ,ymin:ymax]=1

        bones = np.array(self.data3d >= top_threshold)
        aaa = np.array(self.data3d >= heart_threshold)
        aaa = aaa - bones
        logger.debug('pred binary opening')
        aaa=morphology.binary_opening(aaa , iterations=self.iteration()+2).astype(self.segmentation.dtype)
        aaa = morphology.binary_erosion(aaa, iterations=self.iteration())	
        aaa=cc * aaa * mp

        lab , num = label(aaa)
        counts= [0]*(num+1)
        for x in range(1, num+1):
            a = np.sum(np.array(lab == x))
            counts[x] = a
        index= np.argmax(counts)
        aaa = np.array(lab==index)
        logger.debug('pred dilataci')
        aaa = morphology.binary_dilation(aaa, iterations=self.iteration())
        #self.segmentation= aaa
        self.segmentation = self.segmentation + aaa.astype(np.int8)*self.slab['heart']


    def iteration(self, sirka = 5):
        prumer= np.mean(self.voxelsize_mm)
        a = int(sirka / prumer)
        return a

    def orientation(self):
        if(self.segmentation.shape[0]%2==0):
            split1, split2 = np.split(self.segmentation, 2, 0)
            if (np.sum(split1) > np.sum(split2)):
                self.smer=1

            else:
                self.smer=0


        else:
            split1, split2 = np.split(self.segmentation[1:,:,:], 2, 0)
            if(np.sum(split1)>np.sum(split2)):
                self.smer=1

            else:
                self.smer=0



    def volume_count(self, seg_prub):
        labeled_seg , num_seg = label(seg_prub)
        counts= [0]*(num_seg+1)
        for x in range(1, num_seg+1):
            a = np.sum(np.array(labeled_seg == x))
            counts[x] = a
        z, x, y = labeled_seg.shape
        index=labeled_seg[self.iteration()+5,self.iteration()+5,self.iteration()+5]
        counts[index]=0
        index=labeled_seg[self.iteration()+5,self.iteration()+5,y-self.iteration()-5]
        counts[index]=0
        index=labeled_seg[self.iteration()+5,x-self.iteration()-5,self.iteration()+5]
        counts[index]=0
        index=labeled_seg[self.iteration()+5,x-self.iteration()-5,y-self.iteration()-5]
        counts[index]=0
        return counts, labeled_seg





    def lungs_segmentation(self, lungs_threshold = -360):
        seg_prub = np.array(self.data3d <= lungs_threshold)
        seg_prub = morphology.binary_closing(seg_prub , iterations=self.iteration()).astype(self.segmentation.dtype)
        seg_prub = morphology.binary_opening(seg_prub , iterations = 5)
        counts , labeled_seg=self.volume_count(seg_prub)
        #self.segmentation = seg_prub
        #for x in np.nditer(labeled_seg, op_flags=['readwrite']):
        #    if x[...]!=0:890/
        #    	counts[x[...]]=counts[x[...]]+1
        #index=np.argmax(counts) #pozadí
        #counts[index]=0
        index=np.argmax(counts) #jedna nebo obě plíce
        velikost1=counts[index]
        counts[index]=0
        index2=np.argmax(counts)# druhá plíce nebo nečo jiného
        velikost2=counts[index2]
        if (1.0-self.maximal_lung_diff)<= float(velikost2)/velikost1:
            print("plice separované")
        else:
            print("plice neseparované")
            pocet=0
            seg_prub = np.array(self.data3d <= lungs_threshold)
            seg_prub = morphology.binary_closing(seg_prub , iterations=self.iteration()).astype(self.segmentation.dtype)
            seg_prub = morphology.binary_opening(seg_prub , iterations = 5)

            while not (1.0 - self.maximal_lung_diff) <= float(velikost2)/velikost1:
                seg_prub = morphology.binary_erosion(seg_prub,iterations=1)
                counts, labeled_seg = self.volume_count(seg_prub)
                index = np.argmax(counts) #jedna nebo obě plíce
                velikost1=counts[index]
                counts[index]=0
                index2=np.argmax(counts)# druhá plíce nebo nečo jiného
                velikost2=counts[index2]
                pocet=pocet+1
                seg_prub = morphology.binary_dilation(self.segmentation,iterations=pocet).astype(self.segmentation.dtype)
        #self.segmentation = self.segmentation + np.array(labeled_seg==index).astype(np.int8)*self.slab['lungs']
        #self.segmentation = self.segmentation + np.array(labeled_seg==index2).astype(np.int8)*self.slab['lungs']
        plice1 = np.array(labeled_seg==index)
        z,x,y = np.nonzero(plice1)
        m1 = np.max(y)
        if m1<(self.segmentation.shape[1]/2):
            self.segmentation = self.segmentation + np.array(labeled_seg==index).astype(np.int8)*self.slab['llung']
            self.segmentation = self.segmentation + np.array(labeled_seg==index2).astype(np.int8)*self.slab['rlung']
        else:
            self.segmentation = self.segmentation + np.array(labeled_seg==index).astype(np.int8)*self.slab['rlung']
            self.segmentation = self.segmentation + np.array(labeled_seg==index2).astype(np.int8)*self.slab['llung']
        self.orientation()
        if self.smer==1:
            self.segmentation[self.segmentation==self.slab['llung']]=3
            self.segmentation[self.segmentation==self.slab['rlung']]=self.slab['llung']
            self.segmentation[self.segmentation==3]=self.slab['rlung']
        pass



    def _crop(self, data, crinfo):
        """
        Crop data with crinfo
        """
        data = data[crinfo[0][0]:crinfo[0][1], crinfo[1][0]:crinfo[1][1], crinfo[2][0]:crinfo[2][1]]
        return data


    def _crinfo_from_specific_data (self, data, margin):
# hledáme automatický ořez, nonzero dá indexy
        nzi = np.nonzero(data)

        x1 = np.min(nzi[0]) - margin[0]
        x2 = np.max(nzi[0]) + margin[0] + 1
        y1 = np.min(nzi[1]) - margin[0]
        y2 = np.max(nzi[1]) + margin[0] + 1
        z1 = np.min(nzi[2]) - margin[0]
        z2 = np.max(nzi[2]) + margin[0] + 1

# ošetření mezí polí
        if x1 < 0:
            x1 = 0
        if y1 < 0:
            y1 = 0
        if z1 < 0:
            z1 = 0

        if x2 > data.shape[0]:
            x2 = data.shape[0]-1
        if y2 > data.shape[1]:
            y2 = data.shape[1]-1
        if z2 > data.shape[2]:
            z2 = data.shape[2]-1

# ořez
        crinfo = [[x1, x2],[y1,y2],[z1,z2]]
        #dataout = self._crop(data,crinfo)
        #dataout = data[x1:x2, y1:y2, z1:z2]
        return crinfo


    def im_crop(self, im,  roi_start, roi_stop):
        im_out = im[ \
                roi_start[0]:roi_stop[0],\
                roi_start[1]:roi_stop[1],\
                roi_start[2]:roi_stop[2],\
                ]
        return  im_out

    def export(self):
        slab={}
        slab['none'] = 0
        slab['heart']=10
        slab['llung'] = 4
        slab['rlungs'] = 9
        data = {}
        data['version'] = (1,0,0)
        data['data3d'] = self.data3d
        data['crinfo'] = self.crinfo
        data['segmentation'] = self.segmentation
        data['slab'] = slab
        data['voxelsize_mm'] = self.voxelsize_mm
        #import pdb; pdb.set_trace()
        return data




    def visualization(self):
        """
        Run viewer with output data3d and segmentation
        """

        try:
            from imcut.seed_editor_qt import QTSeedEditor
        except:
            logger.warning("Deprecated of pyseg_base as submodule")
            from seed_editor_qt import QTSeedEditor
        from PyQt4.QtGui import QApplication
        import numpy as np
#, QMainWindow
        app = QApplication(sys.argv)
        #pyed = QTSeedEditor(self.data3d, contours=(self.segmentation>0))
        pyed = QTSeedEditor(self.segmentation)
        pyed.exec_()


        #import pdb; pdb.set_trace()


        #pyed = QTSeedEditor(deletemask, mode='draw')
        #pyed.exec_()

        app.exit()






def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(description=
            'Segmentation of bones, lungs and heart.')
    parser.add_argument('-i','--datadir',
            default=None,
            help='path to data dir')
    parser.add_argument('-o','--output',
            default=None,
            help='output file')

    parser.add_argument('-d', '--debug', action='store_true',
            help='run in debug mode')
    parser.add_argument('-ss', '--segspine', action='store_true',
            help='run spine segmentaiton')
    parser.add_argument('-sl', '--seglungs', action='store_true',
            help='run lungs segmentation')
    parser.add_argument('-sh', '--segheart', action='store_true',
            help='run heart segmentation')
    parser.add_argument('-sb', '--segbones', action='store_true',
            help='run bones segmentation')
    parser.add_argument('-exd', '--exampledata', action='store_true',
            help='run unittest')
    parser.add_argument('-so', '--show_output', action='store_true',
            help='Show output data in viewer')
    args = parser.parse_args()



    if args.debug:
        logger.setLevel(logging.DEBUG)


    if args.exampledata:

        args.dcmdir = '../sample_data/liver-orig001.raw'

#    if dcmdir == None:

    #else:
    #dcm_read_from_dir('/home/mjirik/data/medical/data_orig/46328096/')
    #data3d, metadata = dcmr.dcm_read_from_dir(args.dcmdir)

    data3d , metadata = io3d.datareader.read(args.datadir, dataplus_format=False)

    sseg = SupportStructureSegmentation(data3d = data3d,
            voxelsize_mm = metadata['voxelsize_mm'],
            )


    #sseg.orientation()
    if args.segbones:
        sseg.bone_segmentation()
    if args.segspine:
        sseg.spine_segmentation()
    if args.seglungs or args.segheart:
    	sseg.lungs_segmentation()
    if args.segheart:
    	sseg.heart_segmentation()


    sseg.resize_back_to_orig()
    #print("Data size: " + str(data3d.nbytes) + ', shape: ' + str(data3d.shape) )

    #igc = pycut.ImageGraphCut(data3d, zoom = 0.5)
    #igc.interactivity()


    #igc.make_gc()
    #igc.show_segmentation()

    # volume
    #volume_mm3 = np.sum(oseg.segmentation > 0) * np.prod(oseg.voxelsize_mm)


    #pyed = sed3.sed3(oseg.data3d, contour = oseg.segmentation)
    #pyed.show()

    if args.show_output:
        sseg.visualization()

    #savestring = raw_input ('Save output data? (y/n): ')
    #sn = int(snstring)
    if args.output is not None: # savestring in ['Y','y']:
        import misc

        data = sseg.export()

        misc.obj_to_file(data, args.output, filetype = 'pickle')
    #output = segmentation.vesselSegmentation(oseg.data3d, oseg.orig_segmentation)


if __name__ == "__main__":
    main()



# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
List of functions to run
"""
import logging
logger = logging.getLogger(__name__)


class Runner:
    """
    Create list of functions to run. It can parse a list of functions names
    """
    def __init__(self, main_object=None):
        self.run_list = []
        self.main_object = main_object

        pass

    def __get_function_args_from_input(self, fcn, args, kwargs):
        if type(fcn) in (tuple, list):
            fcn, args, kwargs = fcn
        return fcn, args, kwargs

    def __get_function_from_input_string(self, fcn):

        if type(fcn) is str:
            if self.main_object is not None and hasattr(self.main_object, fcn):
                fcn = getattr(self.main_object, fcn)

                pass
            else:
                raise ValueError("Function '{}' not found in given object.".format(fcn))
        return fcn

    def append(self, fcn, *args, **kwargs):
        """

        :param fcn: function or string name of function or list with [fcn, args, kwargs]
        :param args:
        :param kwargs:
        :return:
        """
        fcn, args, kwargs = self.__get_function_args_from_input(fcn, args, kwargs)
        fcn = self.__get_function_from_input_string(fcn)
        self.run_list.append((fcn, args, kwargs))

    def insert(self, i, fcn, *args, **kwargs):
        """

        :param fcn: function or string name of function or list with [fcn, args, kwargs]
        :param args:
        :param kwargs:
        :return:
        """
        fcn, args, kwargs = self.__get_function_args_from_input(fcn, args, kwargs)
        fcn = self.__get_function_from_input_string(fcn)
        self.run_list.insert(i, (fcn, args, kwargs))

    def extend(self, functions):
        """

        :param functions: list of function or string name of function or list with [fcn, args, kwargs]
        ["function1", "function2", ["function3", [arg1, arg2], {"kwarg1":5, "kwarg2": None}, "function3"]
        :return:
        """
        for fcn in functions:
            self.append(fcn)

    def run(self):
        for fcn_with_params in self.run_list:
            fcn, args, kwargs = fcn_with_params
            fcn(*args, **kwargs)



import numpy as np
import time
import timeit


data = (np.random.random([300,300,300])*1000).astype(np.int16)
data2 = []
ta = []
tb = []

for j in range (0,100):
    t0 = time.time()
    for i in range(1,3000):
        data2 = data[10,:,:]
    t1 = time.time()
    ta.append(t1-t0)
    print('cas A: ', t1 - t0)





    data2 = []
    t0 = time.time()
    for i in range(1,3000):
        data2 = data[:,:,10]
    t1 = time.time()

    tb.append(t1-t0)
    print('cas B: ', t1 - t0)

print('sum A: ', np.sum(ta))
print('sum B: ', np.sum(tb))

#import pdb; pdb.set_trace()


#print 'A ', timeit.timeit('data2=data[:,:,1]', number=1000)
#print 'B ', timeit.timeit('data2=data[1,:,:]', number=1000)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@mjirik-Latitude-E6520>
#
# Distributed under terms of the MIT license.

"""

"""

import logging
logger = logging.getLogger(__name__)
import argparse
import os

import PyQt4
import PyQt4.QtCore
import PyQt4.QtGui
from PyQt4.QtGui import QWidget, QGridLayout, QSpinBox, QLineEdit, QCheckBox,\
        QComboBox, QTextEdit, QDialog, QMainWindow, QDoubleSpinBox, QLabel, \
        QPushButton
import sys
if sys.version_info.major == 2:
    from PyQt4.QtCore import QString
else:
    Qstring = str

from .pyqtconfig import ConfigManager

class LisaConfigWindow(QDialog):
# class LisaConfigWindow(QMainWindow):

    def __init__(self, config_in, ncols=2):
        super(LisaConfigWindow, self).__init__()

        self.setWindowTitle('Lisa Config')
        self.config = ConfigManager()
        self.ncols = ncols
# used with other module. If it is set, lisa config is deleted
        self.reset_config = False

        CHOICE_A = "{pairwise_alpha_per_mm2: 45, return_only_object_with_seeds: true}"
        # CHOICE_A = 23

        CHOICE_B = 2
        CHOICE_C = 3
        CHOICE_D = 4

        map_dict = {
            'Graph-Cut': CHOICE_A,
            'Choice B': CHOICE_B,
            'Choice C': CHOICE_C,
            'Choice D': CHOICE_D,
        }
        config_def = {
            'working_voxelsize_mm': 2.0,
            'save_filetype': 'pklz',
            # 'manualroi': True,
            'segparams': CHOICE_A,
        }
        
        config_def.update(config_in)
        
        self.config.set_defaults(config_def)

        gd = QGridLayout()
        gd_max_i = 0
        for key, value in config_in.items():
            if type(value) is int:
                sb = QSpinBox()
                sb.setRange(-100000, 100000)
            elif type(value) is float:
                sb = QDoubleSpinBox()
            elif type(value) is str:
                sb = QLineEdit()
            elif type(value) is bool:
                sb = QCheckBox()
            else:
                logger.error("Unexpected type in config dictionary")

            row = gd_max_i / self.ncols
            col = (gd_max_i % self.ncols) * 2

            gd.addWidget(QLabel(key),row, col +1) 
            gd.addWidget(sb, row, col + 2)
            self.config.add_handler(key, sb)
            gd_max_i += 1




        # gd.addWidget(QLabel("save filetype"), 1, 1)
        # te = QLineEdit()
        # gd.addWidget(te, 1, 2)
        # self.config.add_handler('save_filetype', te)
        #
        # gd.addWidget(QLabel("segmentation parameters"), 2, 1)
        # te = QLineEdit()
        # gd.addWidget(te, 2, 2)
        # self.config.add_handler('segparams', te)

        # cb = QCheckBox()
        # gd.addWidget(cb, 2, 2)
        # self.config.add_handler('active', cb)

        # gd.addWidget(QLabel("segmentation parameters"), 3, 1)
        # cmb = QComboBox()
        # cmb.addItems(map_dict.keys())
        # gd.addWidget(cmb, 3, 2)
        # self.config.add_handler('segparams', cmb, mapper=map_dict)

        self.current_config_output = QTextEdit()
        # rid.setColumnMinimumWidth(3, logo.width()/2)
        text_col = (self.ncols * 2) + 3
        gd.setColumnMinimumWidth(text_col, 500)
        gd.addWidget(self.current_config_output, 1, text_col, (gd_max_i/2)-1, 1)

        btn_reset_config = QPushButton("Reset and quit", self)
        btn_reset_config.clicked.connect(self.btnResetConfig)
        gd.addWidget(btn_reset_config, 0, text_col)

        btn_save_config = QPushButton("Save and quit", self)
        btn_save_config.clicked.connect(self.btnSaveConfig)
        gd.addWidget(btn_save_config, (gd_max_i / 2), text_col)

        self.config.updated.connect(self.show_config)

        self.show_config()

        # my line
        self.setLayout(gd)

        # self.window = QWidget()
        # self.window.setLayout(gd)
        # self.setCentralWidget(self.window)
    def add_grid_line(self, key, value):
        pass

    def get_config_as_dict(self):
        dictionary = self.config.as_dict()
        for key, value in dictionary.items():
            from PyQt4.QtCore import pyqtRemoveInputHook
            pyqtRemoveInputHook()
            # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT
            if type(value) == QString:
                value = str(value)
            dictionary[key] = value

        return dictionary

    def btnResetConfig(self, event=None):
        self.reset_config = True
        self.close()

    def btnSaveConfig(self, event=None):
        self.reset_config = False
        self.close()


    def show_config(self):
        text = str(self.get_config_as_dict())
        self.current_config_output.setText(text)

def configGui(cfg):
    import yaml
    # convert values to json
    isconverted = {}
    for key, value in cfg.items():
        if type(value) in (str, int, float, bool):

            isconverted[key] = False
            if type(value) is str:
                pass

        else:
            isconverted[key] = True
            cfg[key] = yaml.dump(value, default_flow_style=True)

    w = LisaConfigWindow(cfg)
    w.exec_()
    if w.reset_config:
        return None
    
    newcfg = w.get_config_as_dict()
    for key, value in newcfg.items():
        if isconverted[key]:
            newcfg[key] = yaml.load(value)


    return newcfg

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)



if __name__ == "__main__":
    main()



#! /usr/bin/python
# -*- coding: utf-8 -*-
from PIL import Image
import ctypes
import math
import numpy as np
import scipy as sp
import sed3

import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/lbp/"))

import lbpLibrary as lbpLib



def segmentation(data3d, segmentation, params):
    """ Texture analysis by LBP algorithm.
    data: CT (or MRI) 3D data
    segmentation: labeled image with same size as data where label:
    1 mean liver pixels,
    -1 interesting tissuse (bones)
    0 otherwise
    """
    
    MAXPATTERNS = 10

    slices = data3d.shape[2]
    data3d = data3d.astype(np.int16)
    
    params[2] = 0
    data3d = data3d * segmentation;
    data3d = sp.ndimage.filters.gaussian_filter(data3d,params*2.5)  
    pyed = sed3.sed3(data3d)
    pyed.show()
    
    lbp = lbpLib.loadLbpLibrary()    

    lbpReftmp = (ctypes.c_long * 256)()
    lbpRef = np.zeros([MAXPATTERNS,256])

    actualpatterns = 0

    # vytvoreni referencnich LBP obrazu
    for j in range(15,data3d.shape[0]-16):
        if (actualpatterns == MAXPATTERNS) :
            break
        for k in range(15,data3d.shape[1]-16): 
            if (actualpatterns == MAXPATTERNS) :
                break
            if ((pyed.seeds[j,k,pyed.actual_slice] == 1)):                
                lbpReftmp = lbpLib.realTimeLbpImNp(lbp, data3d[j-15:j+16,k-15:k+16,pyed.actual_slice])
                for z in range(256):
                    lbpRef[actualpatterns, z] = lbpReftmp[z]
                print(lbpRef[actualpatterns,])
                actualpatterns += 1                   

    # uprava velikosti dat
    for i in range(20):
        h2 = data3d.shape[0] - i*16
        if (h2 <= 0):
            h2 = i*16
            break
    for i in range(20):
        w2 = data3d.shape[1] - i*16
        if (w2 <= 0):
            w2 = i*16
            break    
    # obdelnikova data prevedeme na ctverec 
    if (w2 >=  h2 ):     
        h2 = w2
    else :
        w2 = h2
    numOfPartsX = (w2) / 16
    numOfPartsY = (h2) / 16   
    minMaxData = np.zeros([h2,w2,data3d.shape[2]],dtype = np.float)
    lbpSlice = np.zeros([h2,w2])
      
    # hlavni cyklus vypocitavajici odezvy
    for i in range(slices):
        tempData = np.zeros([h2,w2],dtype = np.float)
        tempData[0:data3d.shape[0],0:data3d.shape[1]] = data3d[:,:,i].astype(np.float)   
        lbpSlice = lbpLib.realTimeLbpIm2ImNp(lbp, tempData.astype(np.int16))
        lbpRes = lbpLib.lbp2HistsNp(lbp, lbpSlice)
        for j in range(lbpRes.shape[0]):
            minmaxBestVal = 0
            for k in range(MAXPATTERNS):
                minmaxVal = minmax(lbpRes[j,:],lbpRef[k,:])
                if minmaxVal > minmaxBestVal : 
                    minmaxBestVal = minmaxVal
            minMaxData[16*(j/numOfPartsX):16*(j/numOfPartsX)+16,16*(j % numOfPartsX):16*(j % numOfPartsX)+16,i] = minmaxBestVal        

    # zobrazeni vysledku
    pyed2 = sed3.sed3(minMaxData)
    pyed2.show()

    return segmentation

# Funkce pocitajici min max kriterium porovnani dvou histogramu
def minmax(hist1, hist2):
    
    minmaxValue = 0
    minValue = 0
    maxValue = 0
    for i in range(256):
        minValue =  minValue +  min(hist1[i],hist2[i])
        maxValue =  maxValue +  max(hist1[i],hist2[i])
	
    minmaxValue = float(minValue)/float(maxValue+1)
    return minmaxValue



    






#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR%  <>
#
# Distributed under terms of the %LICENSE% license.

"""
Module is used to train liver localizator based on body_navigation module.
Output file is placed into ~/lisa_data/liver.ol.p
"""

import logging

logger = logging.getLogger(__name__)
import numpy as np
import argparse
import glob
import os.path as op
import traceback

import imtools
import imtools.ml
import io3d

import sys
import os
import os.path as op
sys.path.append(op.join(op.dirname(os.path.abspath(__file__)), "../../bodynavigation/"))

from imtools.trainer3d import Trainer3D as OrganLocalizator

def externfv(data3d, voxelsize_mm):        # scale
    return localization_fv(data3d=data3d, voxelsize_mm=voxelsize_mm)


def localization_fv(data3d, voxelsize_mm):        # scale
        fv = []
        # f0 = scipy.ndimage.filters.gaussian_filter(data3d, sigma=3).reshape(-1, 1)
        #f1 = scipy.ndimage.filters.gaussian_filter(data3dr, sigma=1).reshape(-1, 1) - f0
        #f2 = scipy.ndimage.filters.gaussian_filter(data3dr, sigma=5).reshape(-1, 1) - f0
        #f3 = scipy.ndimage.filters.gaussian_filter(data3dr, sigma=10).reshape(-1, 1) - f0
        #f4 = scipy.ndimage.filters.gaussian_filter(data3dr, sigma=20).reshape(-1, 1) - f0
        # position asdfas
        import body_navigation as bn
        ss = bn.BodyNavigation(data3d, voxelsize_mm)
        fd1 = ss.dist_to_lungs().reshape(-1, 1)
        fd2 = ss.dist_to_spine().reshape(-1, 1)
        fd3 = ss.dist_sagittal().reshape(-1, 1)
        fd4 = ss.dist_coronal().reshape(-1, 1)
        fd5 = ss.dist_axial().reshape(-1, 1)
        fd6 = ss.dist_to_surface().reshape(-1, 1)
        fd7 = ss.dist_diaphragm().reshape(-1, 1)

        # f6 = scipy.ndimage.filters.gaussian_filter(data3d, sigma=[20, 1, 1]).reshape(-1, 1) - f0
        # f7 = scipy.ndimage.filters.gaussian_filter(data3d, sigma=[1, 20, 1]).reshape(-1, 1) - f0
        # f8 = scipy.ndimage.filters.gaussian_filter(data3d, sigma=[1, 1, 20]).reshape(-1, 1) - f0


        # print "fv shapes ", f0.shape, fd2.shape, fd3.shape
        fv = np.concatenate([
                # f0,
#                 f1, f2, f3, f4,
                fd1, fd2, fd3, fd4, fd5, fd6, fd7,
                #f6, f7, f8
            ], 1)


        return fv

def localization_intensity_fv(data3d, voxelsize_mm):
    """
    feature vector combine intensity and localization information
    :param data3d:
    :param voxelsize_mm:
    :return:
    """
    fvloc = localization_fv(data3d, voxelsize_mm)
    fv = np.concatenate([
        fvloc,
        data3d.reshape(-1, 1)
        # f0,
        #                 f1, f2, f3, f4,
        #f6, f7, f8
    ], 1)
    return fv




def train_liver_localizator_from_sliver_data(
        output_file="~/lisa_data/liver.ol.p",
        sliver_reference_dir='~/data/medical/orig/sliver07/training/',
        orig_pattern="*orig*[1-9].mhd",
        ref_pattern="*seg*[1-9].mhd"):
    """
    Train liver localization based on spine and lungs from all sliver reference data except number 10 and 20
    :param output_file:
    :param sliver_reference_dir:
    :param orig_pattern:
    :param ref_pattern:
    :return:
    """

    sliver_reference_dir = op.expanduser(sliver_reference_dir)

    orig_fnames = glob.glob(sliver_reference_dir + orig_pattern)
    ref_fnames = glob.glob(sliver_reference_dir + ref_pattern)

    orig_fnames.sort()
    ref_fnames.sort()


    sf = OrganLocalizator()
    vs = []
    hist=[]
    fhs_list = []
    for oname, rname in zip(orig_fnames, ref_fnames):
        print(oname)
        data3d_orig, metadata = io3d.datareader.read(oname, dataplus_format=False)
        vs_mm1 = metadata['voxelsize_mm']
        data3d_seg, metadata = io3d.datareader.read(rname, dataplus_format=False)
        vs_mm = metadata['voxelsize_mm']


        vs.append(vs_mm)

        try:
            sf.add_train_data(data3d_orig, data3d_seg, voxelsize_mm=vs_mm)
        except:
            traceback.print_exc()
            print("problem")
        # fvhn = copy.deepcopy(fvh)
        #fhs_list.append(fvh)


    sf.fit()
    sf.save(op.expanduser(output_file))

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    # parser.add_argument(
    #     '-i', '--inputfile',
    #     default=None,
    #     required=True,
    #     help='input file'
    # )
    parser.add_argument(
        '-o', '--outputfile',
        default="~/lisa_data/liver.ol.p",
        help='output file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)
    train_liver_localizator_from_sliver_data(args.outputfile)


if __name__ == "__main__":
    main()


#! /usr/bin/python
# -*- coding: utf-8 -*-
#definice konstant
BONES = 1290
SPINE = 105
INSIDE_BODY = 5
LUNGS_UP = 400
LUNGS_DOWN = 150	
SPINE_ID = 2
LUNGS_ID = 3

# import funkcí z jiného adresáře
import sys
import os.path
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pycat/"))
sys.path.append(os.path.join(path_to_script, "../extern/pycat/extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../extern/dicom2fem/src"))
#import featurevector

import logging
logger = logging.getLogger(__name__)

import numpy as np
import scipy.ndimage
import argparse

#from PyQt4.QtCore import Qt
#from PyQt4.QtGui import QApplication, QMainWindow, QWidget,\
#     QGridLayout, QLabel, QPushButton, QFrame, QFileDialog,\
#     QFont, QInputDialog, QComboBox, QRadioButton, QButtonGroup

# ----------------- my scripts --------
from lisa import misc
import sed3




class SimpleSegmentation:
    
    def simple_segmentation(self, data3d, voxelsize_mm):
        simple_seg = np.zeros(data3d.shape )
        #definice konvoluční masky
        KONV_MASK = np.ones([10,10,10], float)
        KONV_MASK = KONV_MASK/9.0;


        #definice konvoluční funkce - param a - matice m*n kterou budeme přenásobovat konvoluční maticí, b - Konvoluční maska m*n


        # nalezení kostí
        simple_seg = data3d > BONES
        #simple_seg[(simple_seg.shape[0]/5)*4:simple_seg.shape[0]] = 0

        #nalzení páteře
        spine_finder = scipy.ndimage.filters.convolve((simple_seg).astype(np.int), KONV_MASK)
        #pyed = sed3.sed3(simple_seg)
        #

        simple_seg += ((spine_finder>25)*SPINE_ID)
        pyed = sed3.sed3(simple_seg)
        pyed.show()
        return simple_seg

        
def main():

    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
            description='Module for segmentation of simple anatomical structures')
    parser.add_argument('-i', '--inputfile',
            default='organ.pkl',
            help='path to data dir')

    args = parser.parse_args()

    data = misc.obj_from_file(args.inputfile, filetype = 'pickle')
    data3d = data['data3d']
    voxelsize_mm = data['voxelsize_mm']

    ss = SimpleSegmentation()
    simple_seg = ss.simple_segmentation(data3d, voxelsize_mm)

    #visualization
    pyed = sed3.sed3(data['data3d'], seeds=simple_seg)
    pyed.show()

    # save
    savestring = raw_input('Save output data? (y/n): ')
    if savestring in ['Y', 'y']:
        misc.obj_to_file(data, "resection.pkl", filetype='pickle')


if __name__ == "__main__":
    main()




# ! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)

# import funkcí z jiného adresáře
import sys
import os.path
import shutil

# path_to_script = os.path.dirname(os.path.abspath(__file__))
import unittest

import numpy as np
from nose.plugins.attrib import attr


from lisa import organ_segmentation
import lisa.dataset
import lisa.lisa_data


# nosetests tests/organ_segmentation_test.py:OrganSegmentationTest.test_create_iparams # noqa


class OrganSegmentationTest(unittest.TestCase):

    def generate_data(self):

        img3d = (np.random.rand(30, 30, 30)*10).astype(np.int16)
        seeds = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation[10:25, 4:24, 2:16] = 1
        img3d = img3d + segmentation*20
        seeds[12:18, 9:16, 3:6] = 1
        seeds[19:22, 21:27, 19:21] = 2

        voxelsize_mm = [5, 5, 5]
        metadata = {'voxelsize_mm': voxelsize_mm}
        return img3d, metadata, seeds, segmentation

    # @unittest.skip("in progress")
    def test_sync_paul(self):
        """
        sync with paul account
        """

        path_to_paul = lisa.lisa_data.path('sync','paul')

        if os.path.exists(path_to_paul):
            remove_read_only_flag_on_win32(path_to_paul)
            # handleRemoveReadonly je asi zbytečné
            shutil.rmtree(path_to_paul, onerror=handleRemoveReadonly, ignore_errors=False)
        oseg = organ_segmentation.OrganSegmentation(None)
        oseg.sync_lisa_data('paul','P4ul')


        file_path = lisa.lisa_data.path('sync','paul', 'from_server', 'test.txt')
        logger.debug('file_path %s', file_path)
        self.assertTrue(os.path.exists(file_path))

import errno, os, stat, shutil

def remove_read_only_flag_on_win32(path):
    """
    remove read only flag on windows files recusivelly
    :param path:
    :return:
    """

    platform = sys.platform
    if platform == "win32":
        import win32api, win32con
        for root, dirs, files in os.walk(path):
            for momo in dirs:
                pth = os.path.join(root, momo)
                win32api.SetFileAttributes(pth, win32con.FILE_ATTRIBUTE_NORMAL)
                pass
                # win32api.SetFileAttributes(pth, win32con.FILE_ATTRIBUTE_NORMAL)
            for momo in files:
                pth = os.path.join(root, momo)
                win32api.SetFileAttributes(pth, win32con.FILE_ATTRIBUTE_NORMAL)

def handleRemoveReadonly(func, path, exc):
  excvalue = exc[1]
  if func in (os.rmdir, os.remove) and excvalue.errno == errno.EACCES:
      os.chmod(path, stat.S_IRWXU| stat.S_IRWXG| stat.S_IRWXO| stat.S_IWRITE) # 0777
      os.unlink(path)

      func(path)
  else:
      import traceback
      traceback.print_exc()
      raise Exception("handleRemoveReadonly exception" + str(func))

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest


import numpy as np
import os


from lisa import qmisc
from lisa import misc
import io3d.misc


#

class QmiscTest(unittest.TestCase):
    interactivetTest = False
    # interactivetTest = True

    def test_store_to_SparseMatrix_and_back(self):
        data = np.zeros([4, 4, 4])
        data = np.zeros([4, 4, 4])
        data[1, 0, 3] = 1
        data[2, 1, 2] = 1
        data[0, 1, 3] = 2
        data[1, 2, 0] = 1
        data[2, 1, 1] = 3

        dataSM = qmisc.SparseMatrix(data)

        data2 = dataSM.todense()
        self.assertTrue(np.all(data == data2))

    def test_obj_to_and_from_file_yaml(self):
        testdata = np.random.random([4, 4, 3])
        test_object = {'a': 1, 'data': testdata}

        filename = 'test_obj_to_and_from_file.yaml'
        misc.obj_to_file(test_object, filename, 'yaml')
        saved_object = misc.obj_from_file(filename, 'yaml')

        self.assertTrue(saved_object['a'] == 1)
        self.assertTrue(saved_object['data'][1, 1, 1] == testdata[1, 1, 1])

        os.remove(filename)

    def test_obj_to_and_from_file_pickle(self):
        testdata = np.random.random([4, 4, 3])
        test_object = {'a': 1, 'data': testdata}

        filename = 'test_obj_to_and_from_file.pkl'
        misc.obj_to_file(test_object, filename, 'pickle')
        saved_object = misc.obj_from_file(filename, 'pickle')

        self.assertTrue(saved_object['a'] == 1)
        self.assertTrue(saved_object['data'][1, 1, 1] == testdata[1, 1, 1])

        os.remove(filename)

    # def test_obj_to_and_from_file_exeption(self):
    #    test_object = [1]
    #    filename = 'test_obj_to_and_from_file_exeption'
    #    self.assertRaises(misc.obj_to_file(test_object, filename ,'yaml'))

    def test_obj_to_and_from_file_with_directories(self):
        import shutil
        testdata = np.random.random([4, 4, 3])
        test_object = {'a': 1, 'data': testdata}

        dirname = '__test_write_and_read'
        filename = '__test_write_and_read/test_obj_to_and_from_file.pkl'

        misc.obj_to_file(test_object, filename, 'pickle')
        saved_object = misc.obj_from_file(filename, 'pickle')

        self.assertTrue(saved_object['a'] == 1)
        self.assertTrue(saved_object['data'][1, 1, 1] == testdata[1, 1, 1])

        shutil.rmtree(dirname)

    def test_crop_and_uncrop(self):
        shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo = [[2, 8], [3, 9], [2, 5]]

        img_cropped = qmisc.crop(img_in, crinfo)

        img_uncropped = qmisc.uncrop(img_cropped, crinfo, shape)

        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])

    def test_multiple_crop_and_uncrop(self):
        """
        test combination of multiple crop
        """

        shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo1 = [[2, 8], [3, 9], [2, 5]]
        crinfo2 = [[2, 5], [1, 4], [1, 2]]

        img_cropped = qmisc.crop(img_in, crinfo1)
        img_cropped = qmisc.crop(img_cropped, crinfo2)

        crinfo_combined = qmisc.combinecrinfo(crinfo1, crinfo2)

        img_uncropped = qmisc.uncrop(img_cropped, crinfo_combined, shape)

        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])

    # @unittest.skip("waiting for implementation")
    def test_suggest_filename(self):
        """
        Testing some files. Not testing recursion in filenames. It is situation
        if there exist file0, file1, file2 and input file is file
        """
        filename = "mujsoubor"
        # import ipdb; ipdb.set_trace() # BREAKPOINT
        new_filename = misc.suggest_filename(filename, exists=True)
        self.assertTrue(new_filename == "mujsoubor2")

        filename = "mujsoubor112"
        new_filename = misc.suggest_filename(filename, exists=True)
        self.assertTrue(new_filename == "mujsoubor113")

        filename = "mujsoubor-2.txt"
        new_filename = misc.suggest_filename(filename, exists=True)
        self.assertTrue(new_filename == "mujsoubor-3.txt")

        filename = "mujsoubor-a24.txt"
        new_filename = misc.suggest_filename(filename, exists=False)
        self.assertTrue(new_filename == "mujsoubor-a24.txt")

    # def test_getVersionString(self):
    #     """
    #     """
    #     verstr = qmisc.getVersionString()
    #
    #     self.assertTrue(type(verstr) == str)

    def test_fix_crinfo(self):
        crinfo = [[10, 15], [30, 40], [1, 50]]
        cri_fixed = qmisc.fix_crinfo(crinfo)

        # print crinfo
        # print cri_fixed

        self.assertTrue(cri_fixed[1, 1] == 40)
        self.assertTrue(cri_fixed[2, 1] == 50)


if __name__ == "__main__":
    unittest.main()
